package gr.cite.annotation.query;

import gr.cite.annotation.authorization.AuthorizationFlags;
import gr.cite.annotation.common.enums.IsActive;
import gr.cite.annotation.data.EntityUserEntity;
import gr.cite.tools.data.query.FieldResolver;
import gr.cite.tools.data.query.QueryBase;
import gr.cite.tools.data.query.QueryContext;
import jakarta.persistence.Tuple;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.Predicate;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import java.util.*;

@Component
@RequestScope
public class EntityUserQuery extends QueryBase<EntityUserEntity> {

    private Collection<UUID> ids, entityIds, userIds;

    private Collection<IsActive> isActives;

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    public EntityUserQuery ids(UUID value) {
        this.ids = List.of(value);
        return this;
    }

    public EntityUserQuery ids(UUID... value) {
        this.ids = Arrays.asList(value);
        return this;
    }

    public EntityUserQuery ids(Collection<UUID> values) {
        this.ids = values;
        return this;
    }

    public EntityUserQuery entityIds(UUID value) {
        this.entityIds = List.of(value);
        return this;
    }

    public EntityUserQuery entityIds(UUID... value) {
        this.entityIds = Arrays.asList(value);
        return this;
    }

    public EntityUserQuery entityIds(Collection<UUID> values) {
        this.entityIds = values;
        return this;
    }

    public EntityUserQuery userIds(UUID value) {
        this.userIds = List.of(value);
        return this;
    }

    public EntityUserQuery userIds(UUID... value) {
        this.userIds = Arrays.asList(value);
        return this;
    }

    public EntityUserQuery userIds(Collection<UUID> values) {
        this.userIds = values;
        return this;
    }

    public EntityUserQuery isActive(IsActive value) {
        this.isActives = List.of(value);
        return this;
    }

    public EntityUserQuery isActive(IsActive... value) {
        this.isActives = Arrays.asList(value);
        return this;
    }

    public EntityUserQuery isActive(Collection<IsActive> values) {
        this.isActives = values;
        return this;
    }

    public EntityUserQuery authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    protected Boolean isFalseQuery() {
        return false;
    }

    @Override
    protected Class<EntityUserEntity> entityClass() {
        return EntityUserEntity.class;
    }

    @Override
    protected <X, Y> Predicate applyFilters(QueryContext<X, Y> queryContext) {
        List<Predicate> predicates = new ArrayList<>();
        if (this.ids != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(EntityUserEntity._id));
            for (UUID item : this.ids)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.entityIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(EntityUserEntity._entityId));
            for (UUID item : this.entityIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.userIds != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(EntityUserEntity._userId));
            for (UUID item : this.userIds)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.isActives != null) {
            CriteriaBuilder.In<IsActive> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(EntityUserEntity._isActive));
            for (IsActive item : this.isActives)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return null;
        }
    }

    @Override
    protected String fieldNameOf(FieldResolver item) {
        return null;
    }

    @Override
    protected EntityUserEntity convert(Tuple tuple, Set<String> columns) {
        return null;
    }
}
